//Author SharpPlus Software Corp.
//Date2004-12-1
//ActionList For .Net Design time components

using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Globalization;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;

namespace SharpPlus.Action.Design
{
	#region ActionList Designer
	/// <summary>
	/// Action List Designer
	/// </summary>
	internal class ActionDesigner: ComponentDesigner
	{
		private DesignerVerb dvExecute;

//		public override void InitializeNonDefault()
//		{
//			base.InitializeNonDefault ();
//            dvExecute=new DesignerVerb("Action Designer", new EventHandler(OnExecute));
//		}

		//Init component can be implemented by override void Initialize(IComponent c) method
		//may be used to replace Application.OnIdle event handler

		private ActionList myActionList;
		public override void Initialize(IComponent component)
		{
			base.Initialize (component);
			myActionList=(ActionList)component;
		}

		public ActionDesigner():base()
		{
            dvExecute=new DesignerVerb("Action Designer", new EventHandler(this.OnExecute));
		}

		public override DesignerVerbCollection Verbs
		{
			get
			{
				DesignerVerbCollection verbs=new DesignerVerbCollection();
				verbs.Add(dvExecute);
				return verbs;
			}
		}

		public override void DoDefaultAction()
		{
			this.OnExecute(null, null);
		}


		private void OnExecute(object sender, System.EventArgs e)
		{
			DsgnForm.ShowActionDesigner(myActionList);
		}

		//support clipboard operations
		public override System.Collections.ICollection AssociatedComponents
		{
			get
			{
				return myActionList.Actions;
			}
		}

	}

	#endregion

	#region ImageIndexEditor

	internal class ImageIndexEditor : UITypeEditor
	{
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			if(context != null && context.Instance != null ) 
			{
				return UITypeEditorEditStyle.DropDown;
			}
			return base.GetEditStyle(context);
		}

		public override bool GetPaintValueSupported(ITypeDescriptorContext context) 
		{
			return true;
		}

		public override void PaintValue(PaintValueEventArgs pe) 
		{
			int imageIndex = -1;	// Do not display an image
			if( pe.Value != null )
			{
				try
				{
					imageIndex = (int)Convert.ToUInt16( pe.Value.ToString() ) ;
				}
				catch
				{
					imageIndex = -1;
				}
			}

			if( pe.Context.Instance == null || imageIndex < 0 )
				return;

			Action act = (Action)pe.Context.Instance;
			ActionList actList = act.ActionList;
			ImageList imageList = actList.ImageList;

			if( imageList == null || 
				imageList.Images.Empty || 
				imageIndex >= imageList.Images.Count )
				return;

			pe.Graphics.DrawImage( imageList.Images[imageIndex], pe.Bounds );
		}

		public override object EditValue(ITypeDescriptorContext context,
			IServiceProvider provider, object value)
		{
			IWindowsFormsEditorService wfes = (IWindowsFormsEditorService)provider.GetService(
				typeof(IWindowsFormsEditorService));

			if( wfes == null || context == null )
				return null;

			Action act = (Action)context.Instance;
			ActionList actList = act.ActionList;
			ImageList imageList = actList.ImageList;

			ImageSelector imageSelector = new ImageSelector( imageList, (int)value, wfes );

			wfes.DropDownControl( imageSelector );

			int imageIndex = -1;
			if( imageSelector.SelectedItems.Count != 0 )
			{
				try
				{
					imageIndex = (int)Convert.ToInt32( imageSelector.SelectedItems[0].Text );
				}
				catch
				{
					imageIndex = -1;
				}
			}
			return imageIndex;
		}

		private class ImageSelector : System.Windows.Forms.ListView
		{
			private ImageList	imageList = null;
			private IWindowsFormsEditorService wfes = null;

			public ImageSelector( ImageList images, int selectedIndex, IWindowsFormsEditorService wfes )
			{
				this.wfes = wfes;
				this.BorderStyle = BorderStyle.None;
				this.View = View.Details;
				this.FullRowSelect = true;
				this.GridLines = false;
				this.Columns.Add("Index", -2, HorizontalAlignment.Left);
				this.HeaderStyle = ColumnHeaderStyle.None;
				this.MultiSelect = false;
				this.Click += new EventHandler( ImageSelector_Click );

				if( images != null )
				{
					// Scale images for the ListView if necessary.
					// You can assign this.imageList=images without this check.

					if( images.ImageSize == SystemInformation.SmallIconSize )
						this.imageList = images;
					else
					{
						this.imageList = new ImageList();
						this.imageList.ImageSize = SystemInformation.SmallIconSize;
						for( int i=0; i < images.Images.Count; i++ )
							this.imageList.Images.Add( images.Images[i] );
					}

					this.SmallImageList = imageList;
					for( int i=0; i < imageList.Images.Count; i++ )
					{
						this.Items.Add( i.ToString(), i );
					}
				}

				this.Items.Insert(0,"[none]");
				if( selectedIndex>=this.Items.Count-1 )
					selectedIndex = -1;

				this.Items[ selectedIndex+1 ].Selected = true;
				this.Items[ selectedIndex+1 ].EnsureVisible();
			}

			protected void ImageSelector_Click(object sender, EventArgs e)
			{
				if( wfes != null )
					wfes.CloseDropDown();
			}
		}
	}
	#endregion

	#region IndexConverter

	internal class IndexConverter : TypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType) 
		{
			if( sourceType == typeof(string) )
			{
				return true;
			}
			return base.CanConvertFrom(context, sourceType);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType) 
		{
			if( destinationType == typeof(string) ) 
				return true;
			return base.CanConvertTo(context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, 
			CultureInfo culture, object value) 
		{
			if( value is string )
			{
				int imageIndex = 0;
				string indexValue = (string)value;
				if( value != null && indexValue.Length > 0 && indexValue != "[none]" )
				{
					try
					{
						imageIndex = (int)Convert.ToUInt16( indexValue );
					}
					catch
					{
						return -1;
					}
				}
				else
					return -1;

				return imageIndex;
			}
			return base.ConvertFrom(context, culture, value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, 
			CultureInfo culture, object value, Type destinationType) 
		{
			if( destinationType == typeof(string) && value is int ) 
			{
				return ( (int)value > -1 ) ? value.ToString() : "[none]";
			}
			return base.ConvertTo(context, culture, value, destinationType);      
		}
	}
	#endregion

	#region CategoryEditor
	/// <summary>
	/// Category's property editor
	/// </summary>
	internal class CategoryEditor: UITypeEditor
	{
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
		{
			if(context != null && context.Instance != null ) 
			{
				return UITypeEditorEditStyle.DropDown;
			}
			return base.GetEditStyle(context);
		}

		public override object EditValue(ITypeDescriptorContext context,
			IServiceProvider provider, object value)
		{
			IWindowsFormsEditorService wfes = (IWindowsFormsEditorService)provider.GetService(
				typeof(IWindowsFormsEditorService));

			if( wfes == null || context == null )
				return null;

			Component component = (Component)context.Instance;
			Action Action=component as Action;
			ActionList actList= Action.ActionList;

			CategoryListBox lbCategory=new CategoryListBox(actList.Actions, Action.Category, wfes);
			wfes.DropDownControl(lbCategory);

			return lbCategory.SelectedCategory;
		}

		private class CategoryListBox:ListBox
		{
			private ActionCollection	actList = null;
			private IWindowsFormsEditorService wfes = null;
			private string _Category="";

			public CategoryListBox(ActionCollection Actions, string Category, IWindowsFormsEditorService wfes)
			{
				this.actList=Actions;
				this.wfes=wfes;
				this.BorderStyle=BorderStyle.None;
				this._Category=Category;
				//new Category
				Hashtable categories=new Hashtable(5);
				foreach (Action AAction in actList)
				{
					if (AAction.Category==string.Empty || categories.ContainsKey(AAction.Category))
						continue;
					categories.Add(AAction.Category, AAction.Category);
				}

				foreach (string key in categories.Values)
				{
					this.Items.Add(key);
				}

			    this.Sort();
				this.Items.Insert(0, "No Category");


				
				//Select original Action
				if (Category=="")
					this.SelectedIndex=0;
				else
					this.SelectedIndex=this.Items.IndexOf(Category);

				this.Click+=new EventHandler(CategoryListBox_Click);
			}

			private void CategoryListBox_Click(object sender, EventArgs e)
			{
				//Select some category
				if (this.SelectedIndex==0)
					_Category="";
				else
					_Category=(string)this.Items[this.SelectedIndex];
				if (this.wfes!=null)
					wfes.CloseDropDown();
			}

			public string SelectedCategory
			{
				get {return _Category;}
			}
		}
	}

	#endregion

	#region ActionEditor
	/// <summary>
	/// ActionEditor is Action's Editor
	/// </summary>
	internal class ActionEditor : UITypeEditor
	{
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			if(context != null && context.Instance != null ) 
			{
				return UITypeEditorEditStyle.DropDown;
			}
			return base.GetEditStyle(context);
		}

#warning Todo:Draw property value

		public override object EditValue(ITypeDescriptorContext context,
			IServiceProvider provider, object value)
		{
			IWindowsFormsEditorService wfes = (IWindowsFormsEditorService)provider.GetService(
				typeof(IWindowsFormsEditorService));

			if( wfes == null || context == null )
				return null;

			ActionList actList= null;
			Component component = (Component)context.Instance;

			IExtenderListService extenderListService = (IExtenderListService)component.Site.GetService(typeof(IExtenderListService));
			if( extenderListService != null )
			{
				IExtenderProvider[] extenders = extenderListService.GetExtenderProviders();
				for( int i=0; i < extenders.Length; i++ )
				{
					if( extenders[i].GetType().FullName == "SharpPlus.Action.ActionList" )
					{
						actList = (ActionList)extenders[i];
					}
				}
			}

			if (actList.Actions.Count==0)
				return null;

			ActionListBox lbActions=new ActionListBox(actList.Actions, (Action)value, wfes);
			wfes.DropDownControl(lbActions);

			return lbActions.SelectedAction;
		}

		private class ActionListBox:ListBox
		{
			private ActionCollection	actList = null;
			private IWindowsFormsEditorService wfes = null;
			private Action _Action=null;

			public ActionListBox(ActionCollection Actions, Action Action, IWindowsFormsEditorService wfes)
			{
				this.actList=Actions;
				this.wfes=wfes;
				this.BorderStyle=BorderStyle.None;
				this._Action=Action;
				this.Items.Add("None");
				foreach (Action AAction in actList)
				{
					this.Items.Add(AAction);
				}
				
				//Select original action
				if (Action==null)
					this.SelectedIndex=0;
				else
					this.SelectedIndex=this.Items.IndexOf(Action);

				this.Click+=new EventHandler(ActionListBox_Click);
			}

			private void ActionListBox_Click(object sender, EventArgs e)
			{
				//select some action
				if (this.SelectedIndex==0)
					_Action=null;
				else
					_Action=(Action)this.Items[this.SelectedIndex];
				if (this.wfes!=null)
					wfes.CloseDropDown();
			}

			public Action SelectedAction
			{
				get {return _Action;}
			}
		}
		#endregion

	}

}
