//AuthorSharpPlus Software Corp.
//Date2004-11-29
//ActionList menuitem, button, toolbar button's binder

using System;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace SharpPlus.Action
{
	/// <summary>
	/// ExtenderManager contains an array of custom ActionExtender classes.
	/// </summary>
	public class ExtenderManager
	{
		private static Hashtable ActionExtenders=new Hashtable(5);
		private static ExtenderManager _Manager;
		private ExtenderManager()
		{
			//Register ActionExtender
			RegisterExtender(typeof(Button),new ButtonExtender());
			RegisterExtender(typeof(MenuItem), new MenuItemExtender());
			RegisterExtender(typeof(ToolBarButton), new ToolbarButtonExtender());
		}

		internal static ActionExtender GetActionExtender(object extendee)
		{
			if (_Manager==null)
				_Manager=new ExtenderManager();

			Object o=ActionExtenders[extendee.GetType()];
			if (o==null)
				return null;
			return o as ActionExtender;
		}

		/// <summary>
		/// To install your own action extender classes so that custom client controls  can be binded to action, call RegisterExtender to register them.
		/// </summary>
		/// <param name="ctrlType">The ctrlType parameter specifies the type of custom client control to be binded to action</param>
		/// <param name="extender">The extender parameter specifies the extender class that implements the ActionExtender interface</param>
		public static void RegisterExtender(Type ctrlType, ActionExtender extender)
		{
			ActionExtenders.Add(ctrlType, extender);
		}
	}

	/// <summary>
	/// ActionExtender interface can be used to bind action to custom client control.
	/// for example, if you want to bind action to third party controls, you can write a custom ActionExtender class which implements ActionExtender interface.
	/// The predefined ActionExtender classes can help you bind action to menuitems, buttons and toolbar buttons.
	/// </summary>
	public interface ActionExtender
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetCaption(object extendee, string value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetEnabled(object extendee, bool value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetVisible(object extendee, bool value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetTooltip(object extendee, string value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetShortcut(object extendee, Shortcut value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetShowShortcut(object extendee, bool value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		void SetChecked(object extendee, bool value);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="ImageIndex"></param>
		void SetImageIndex(object extendee, int ImageIndex);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		void BindTarget(object target, Action Action);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		void UnBindTarget(object target, Action Action);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		void EndInit(object target, Action Action);
	}

	/// <summary>
	/// base class which implement the ActionExtender interface.
	/// All ActionExtender class should be inherited from it.
	/// </summary>
	[DesignTimeVisible(false), ToolboxItem(false)]
	public abstract class BaseExtender:Component,ActionExtender
	{
		internal static ToolTip _Tooltip=new ToolTip();
		/// <summary>
		/// 
		/// </summary>
		public BaseExtender()
		{

		}
		#region ActionExtender Member

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetCaption(object extendee, string value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetEnabled(object extendee, bool value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetVisible(object extendee, bool value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetTooltip(object extendee, string value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetShortcut(object extendee, Shortcut value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public virtual void SetShowShortcut(object extendee, bool value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendd"></param>
		/// <param name="value"></param>
		public virtual void SetChecked(object extendd, bool value)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="ImageIndex"></param>
		public virtual void SetImageIndex(object extendee, int ImageIndex)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public virtual void BindTarget(object target, Action Action)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public virtual void UnBindTarget(object target, Action Action)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public virtual void EndInit(object target, Action Action)
		{
		}

		#endregion
	}

	/// <summary>
	/// The ButtonExtender class is used to bind actions to buttons.
	/// </summary>
	public class ButtonExtender:BaseExtender
	{
		#region ActionExtender Member

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetCaption(object extendee, string value)
		{
			((Button)extendee).Text=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetEnabled(object extendee, bool value)
		{
			((Button)extendee).Enabled=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetVisible(object extendee, bool value)
		{
			((Button)extendee).Visible=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetTooltip(object extendee, string value)
		{
			_Tooltip.SetToolTip(extendee as Control, value);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="ImageIndex"></param>
		public override void SetImageIndex(object extendee, int ImageIndex)
		{
			((Button)extendee).ImageIndex=ImageIndex;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void BindTarget(object target, Action Action)
		{
			Button btnTarget=(Button)target;
			btnTarget.Text=Action.Caption;
			btnTarget.Enabled=Action.Enabled;
			btnTarget.Visible=Action.Visible;
			btnTarget.ImageIndex=Action.ImageIndex;
			if (btnTarget.ImageList==null && Action.ActionList!=null)
				btnTarget.ImageList=Action.ActionList.ImageList;
			if (!this.DesignMode)
				_Tooltip.SetToolTip(btnTarget,Action.Tooltip);
			btnTarget.Click-=new EventHandler(Action.DoExecute);
			btnTarget.Click+=new EventHandler(Action.DoExecute);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void UnBindTarget(object target, Action Action)
		{
			Button btnTarget=(Button)target;
			if (!btnTarget.Site.DesignMode)
				_Tooltip.SetToolTip(btnTarget,"");
			btnTarget.Click-=new EventHandler(Action.DoExecute);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void EndInit(object target, Action Action)
		{
			Button btnTarget=(Button)target;
			if (btnTarget.ImageList==null && Action.ActionList!=null)
				btnTarget.ImageList=Action.ActionList.ImageList;
		}


		#endregion
	}

	/// <summary>
	/// The MenuItemExtender class is used to bind actions to menuitems.
	/// </summary>
	public class MenuItemExtender:BaseExtender
	{
		#region ActionExtender Member

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetCaption(object extendee, string value)
		{
			MenuItem item=(MenuItem)extendee;
			item.Text=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetEnabled(object extendee, bool value)
		{
			MenuItem item=(MenuItem)extendee;
			item.Enabled=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetVisible(object extendee, bool value)
		{
			((MenuItem)extendee).Visible=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetShortcut(object extendee, Shortcut value)
		{
			((MenuItem)extendee).Shortcut=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetShowShortcut(object extendee, bool value)
		{
			((MenuItem)extendee).ShowShortcut=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetChecked(object extendee, bool value)
		{
			((MenuItem)extendee).Checked=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void BindTarget(object target, Action Action)
		{
			MenuItem item=(MenuItem)target;
			item.Text=Action.Caption;
			item.Enabled=Action.Enabled;
			item.Visible=Action.Visible;
			item.Shortcut=Action.ShortCut;
			item.Checked=Action.Checked;
			item.ShowShortcut=Action.ShowShortcut;
			item.Click-=new EventHandler(Action.DoExecute);
			item.Click+=new EventHandler(Action.DoExecute);

			//Bind to draw icon event
			BindEvent(item, Action);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void UnBindTarget(object target, Action Action)
		{
			MenuItem item=(MenuItem)target;
			item.Click-=new EventHandler(Action.DoExecute);

			item.MeasureItem	-= new MeasureItemEventHandler( Action.OnMeasureItem ) ;
			item.DrawItem		-= new DrawItemEventHandler( Action.OnDrawItem ) ;
			item.OwnerDraw		= false;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void EndInit(object target, Action Action)
		{
			MenuItem item=(MenuItem)target;
			BindEvent(item, Action);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="item"></param>
		/// <param name="Action"></param>
		private void BindEvent(MenuItem item, Action Action)
		{
			item.MeasureItem	-= new MeasureItemEventHandler( Action.OnMeasureItem ) ;
			item.DrawItem		-= new DrawItemEventHandler( Action.OnDrawItem ) ;

			if (Action.ActionList==null)
				return;
			if (Action.ActionList.ImageList==null)
				return;

			item.MeasureItem	+= new MeasureItemEventHandler( Action.OnMeasureItem ) ;
			item.DrawItem		+= new DrawItemEventHandler( Action.OnDrawItem ) ;
			item.OwnerDraw		= true;
		}


		#endregion
	}

	/// <summary>
	/// The ToolbarButtonExtender class is used to bind actions to toolbar buttons.
	/// </summary>
	public class ToolbarButtonExtender:BaseExtender
	{
		#region ActionExtender Member

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetCaption(object extendee, string value)
		{
			ToolBarButton btn=(ToolBarButton)extendee;
			btn.Text=value;			
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetEnabled(object extendee, bool value)
		{
			ToolBarButton btn=(ToolBarButton)extendee;
			btn.Enabled=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetVisible(object extendee, bool value)
		{
			((ToolBarButton)extendee).Visible=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="value"></param>
		public override void SetTooltip(object extendee, string value)
		{
			((ToolBarButton)extendee).ToolTipText=value;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <param name="ImageIndex"></param>
		public override void SetImageIndex(object extendee, int ImageIndex)
		{
			((ToolBarButton)extendee).ImageIndex=ImageIndex;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void BindTarget(object target, Action Action)
		{
			ToolBarButton btn=(ToolBarButton)target;
			if (Action.ShowToolBarButtonCaption)
			  btn.Text=Action.Caption;
			btn.Enabled=Action.Enabled;
			btn.Visible=Action.Visible;
			btn.ToolTipText=Action.Tooltip;
			btn.ImageIndex=Action.ImageIndex;
			BindEvent(btn, Action);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void UnBindTarget(object target, Action Action)
		{
			ToolBarButton btn=(ToolBarButton)target;
			if (btn.Parent==null)
				return;
			btn.Parent.ButtonClick-=new ToolBarButtonClickEventHandler(Action.ActionList.DoToolbarExecute);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="target"></param>
		/// <param name="Action"></param>
		public override void EndInit(object target, Action Action)
		{
			ToolBarButton btn=(ToolBarButton)target;
			BindEvent(btn, Action);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="btn"></param>
		/// <param name="Action"></param>
		private void BindEvent(ToolBarButton btn, Action Action)
		{
			if (btn.Parent==null)
				return;
			if (btn.Parent.ImageList==null && Action.ActionList!=null)
				btn.Parent.ImageList=Action.ActionList.ImageList;
			btn.Parent.ButtonClick-=new ToolBarButtonClickEventHandler(Action.ActionList.DoToolbarExecute);
			btn.Parent.ButtonClick+=new ToolBarButtonClickEventHandler(Action.ActionList.DoToolbarExecute);
		}
		#endregion

	}

}
